import os
import c4d
from c4d.modules import mograph as mo

# Be sure to use a unique ID obtained from www.plugincafe.com
PLUGIN_ID = 1060757


class LookAtCamera(c4d.plugins.TagData):
    """Look at Camera"""
    
    def Init(self, node: c4d.GeListNode):
        """Called when Cinema 4D Initialize the TagData (used to define, default values).

        Args:
            node (c4d.GeListNode): The instance of the TagData.

        Returns:
            True on success, otherwise False.
        """
        self.InitAttr(node, int, c4d.CLONER_INDEX)
        self.InitAttr(node, c4d.BaseObject, c4d.LINK_NODE)
        self.InitAttr(node, bool, c4d.STIKY_ON_CLONER)
        
        node[c4d.CLONER_INDEX] = 0
        node[c4d.LINK_NODE] = None
        node[c4d.STIKY_ON_CLONER] = False

        return True
    
    def Execute(self, tag, doc, op, bt, priority, flags):
        """Called by Cinema 4D at each Scene Execution, this is the place where calculation should take place.

        Args:
            tag (c4d.BaseTag): The instance of the TagData.
            doc (c4d.documents.BaseDocument): The host document of the tag's object.
            op (c4d.BaseObject): The host object of the tag.
            bt (c4d.threading.BaseThread): The Thread that execute the this TagData.
            priority (EXECUTIONPRIORITY): Information about the execution priority of this TagData.
            flags (EXECUTIONFLAGS): Information about when this TagData is executed.
        """
        # obj link
        link_node = tag[c4d.LINK_NODE]

        # check op is a cloner
        if op.GetType() != 1018544:
            return c4d.EXECUTIONRESULT_OK
        
        # get data
        md = mo.GeGetMoData(op)
        if not md:
            return c4d.EXECUTIONRESULT_OK
        
        # certain mg of index data
        md_mg = md.GetArray(c4d.MODATA_MATRIX)
        user_index = tag[c4d.CLONER_INDEX]
        index_mg = md_mg[user_index]

        if not index_mg:
            return c4d.EXECUTIONRESULT_OK

        global_index_mg = link_node.GetMg() * index_mg

        link_node.SetMg(global_index_mg)

        link_node.Message(c4d.MSG_UPDATE)
        #c4d.EventAdd()

        return c4d.EXECUTIONRESULT_OK

if __name__ == "__main__":
    # Retrieves the icon path
    directory, _ = os.path.split(__file__)
    fn = os.path.join(directory, "res", "icon.png")

    # Creates a BaseBitmap
    bmp = c4d.bitmaps.BaseBitmap()
    if bmp is None:
        raise MemoryError("Failed to create a BaseBitmap.")

    # Init the BaseBitmap with the icon
    if bmp.InitWith(fn)[0] != c4d.IMAGERESULT_OK:
        raise MemoryError("Failed to initialize the BaseBitmap.")

    c4d.plugins.RegisterTagPlugin(id=PLUGIN_ID,
                                  str="Taligncloner",
                                  info=c4d.TAG_EXPRESSION | c4d.TAG_VISIBLE,
                                  g=LookAtCamera,
                                  description="Taligncloner",
                                  icon=bmp)
